@@ -43,6 +43,7 @@ iBool       isSimpleCharMode_Video  = iFalse;
 static iBlock *runSystemCommand_(const char *cmd) {
     iBlock *result = new_Block(0);
     FILE *p = popen(cmd, "r");
+    if (!p) return result;
     for (;;) {
         int c = fgetc(p);
         if (c == EOF) break;
@@ -53,6 +54,7 @@ static iBlock *runSystemCommand_(const char *cmd) {
 }
 
 iInt2 SDL_QueryTerminalSize(void) {
+    /* TODO: Curses may provide ROWS and COLS; check environment, too. */
     iBlock *output = runSystemCommand_("stty size");
     iInt2 size = zero_I2();
     sscanf(cstr_Block(output), "%d %d", &size.y, &size.x);
@@ -63,7 +65,7 @@ iInt2 SDL_QueryTerminalSize(void) {
 }
 
 static void postControlC_(int s) {
-    SDL_SendKeyboardPress(SDLK_c, KMOD_CTRL);    
+    SDL_SendKeyboardPress(SDLK_c, KMOD_CTRL);
 }
 
 static void postQuit_(int s) {
@@ -119,7 +121,7 @@ SDL_Window *SDL_CreateWindow(const char *title, int x, int y, int w, int h, int
     wrefresh(cwin);
     d->w = size.x;
     d->h = size.y;
-    currentWindow_Video = d;   
+    currentWindow_Video = d;
     if (~d->flags & SDL_WINDOW_HIDDEN) {
         SDL_PushWindowEvent(d, SDL_WINDOWEVENT_EXPOSED);
         SDL_PushWindowEvent(d, SDL_WINDOWEVENT_SHOWN);
@@ -209,19 +211,19 @@ void SDL_GetDisplayBounds(int index, SDL_Rect *bounds) {
 void SDL_GetDisplayUsableBounds(int index, SDL_Rect *bounds) {
     if (index == 0) {
         *bounds = (SDL_Rect){ 0, 0, currentWindow_Video->w, currentWindow_Video->h };
-    }    
+    }
 }
 
 void SDL_SetWindowTitle(SDL_Window *d, const char *title) {
-    
+
 }
 
 void SDL_SetWindowFullscreen(SDL_Window *d, int mode) {
-    
+
 }
 
 void SDL_SetWindowInputFocus(SDL_Window *d) {
-    
+
 }
 
 void SDL_ResizeWindow(SDL_Window *d, int w, int h) {
