@@ -54,8 +54,8 @@ void init_Url(iUrl *d, const iString *text) {
     static iRegExp *authPattern_;
     if (!urlPattern_) {
         urlPattern_  = new_RegExp("^(([-.+a-z0-9]+):)?(//([^/?#]*))?"
-                                 "([^?#]*)(\\?([^#]*))?(#(.*))?",
-                                 caseInsensitive_RegExpOption);
+                                  "([^?#]*)(\\?([^#]*))?(#(.*))?",
+                                  caseInsensitive_RegExpOption);
         authPattern_ = new_RegExp("(([^@]+)@)?(([^:\\[\\]]+)"
                                   "|(\\[[0-9a-f:]+\\]))(:([0-9]+))?",
                                   caseInsensitive_RegExpOption);
@@ -73,6 +73,7 @@ void init_Url(iUrl *d, const iString *text) {
         /* Check if the authority contains a port. */
         init_RegExpMatch(&m);
         if (matchRange_RegExp(authPattern_, d->host, &m)) {
+            d->user = capturedRange_RegExpMatch(&m, 2);
             d->host = capturedRange_RegExpMatch(&m, 3);
             d->port = capturedRange_RegExpMatch(&m, 7);
         }
@@ -110,6 +111,9 @@ uint16_t port_Url(const iUrl *d) {
     else if (equalCase_Rangecc(d->scheme, "spartan")) {
         port = 300;
     }
+    else if (equalCase_Rangecc(d->scheme, "guppy")) {
+        port = 6775;
+    }
     else if (equalCase_Rangecc(d->scheme, "https")) {
         port = 443;
     }
@@ -232,6 +236,18 @@ iRangecc urlHost_String(const iString *d) {
     return url.host;
 }
 
+iRangecc urlDirectory_String(const iString *d) {
+    iUrl parts;
+    init_Url(&parts, d);
+    if (size_Range(&parts.path) > 0 && !endsWith_Rangecc(parts.path, "/")) {
+        /* Remove the last path segment. */
+        while (parts.path.end > parts.path.start && parts.path.end[-1] != '/') {
+            parts.path.end--;
+        }
+    }
+    return parts.path;
+}
+
 uint16_t urlPort_String(const iString *d) {
     iUrl url;
     init_Url(&url, d);
@@ -244,11 +260,13 @@ iRangecc urlUser_String(const iString *d) {
         userPats_[0] = new_RegExp("~([^/?]+)", 0);
         userPats_[1] = new_RegExp("/users/([^/?]+)", caseInsensitive_RegExpOption);
     }
+    iUrl url;
+    init_Url(&url, d);
     iRegExpMatch m;
-    init_RegExpMatch(&m);
     iRangecc found = iNullRange;
     iForIndices(i, userPats_) {
-        if (matchString_RegExp(userPats_[i], d, &m)) {
+        init_RegExpMatch(&m);
+        if (matchRange_RegExp(userPats_[i], url.path, &m)) {
             found = capturedRange_RegExpMatch(&m, 1);
         }
     }
@@ -272,6 +290,12 @@ iRangecc urlRoot_String(const iString *d) {
     return (iRangecc){ constBegin_String(d), rootEnd };
 }
 
+iRangecc urlPath_String(const iString *d) {
+    iUrl url;
+    init_Url(&url, d);
+    return url.path;
+}
+
 const iBlock *urlThemeSeed_String(const iString *url) {
     if (equalCase_Rangecc(urlScheme_String(url), "file")) {
         return collect_Block(new_Block(0));
@@ -328,7 +352,7 @@ void urlDecodePath_String(iString *d) {
     iString *decoded = new_String();
     appendRange_String(decoded, (iRangecc){ constBegin_String(d), url.path.start });
     iString *path    = newRange_String(url.path);
-    iString *decPath = urlDecodeExclude_String(path, "%?/#"); /* don't decode reserved path chars */
+    iString *decPath = urlDecodeExclude_String(path, ":%?/#"); /* don't decode reserved path chars */
     append_String(decoded, decPath);
     delete_String(decPath);
     delete_String(path);
@@ -358,6 +382,28 @@ void urlEncodePath_String(iString *d) {
     delete_String(encoded);
 }
 
+iString *withUrlParameters_String(const iString *d, ...) {
+    iUrl url;
+    init_Url(&url, d);
+    iString *updated = new_String();
+    appendRange_String(updated, (iRangecc){ constBegin_String(d), url.path.end });
+    va_list args;
+    for (va_start(args, d);;) {
+        const char *key = va_arg(args, const char *);
+        if (!key) break;
+        const char *value = va_arg(args, const char *);
+        appendCStr_String(updated, ";");
+        appendCStr_String(updated, key);
+        if (value) {
+            appendCStr_String(updated, "=");
+            appendCStr_String(updated, value);
+        }
+    }
+    appendRange_String(updated, (iRangecc){ url.path.end, constEnd_String(d) });
+    va_end(args);
+    return updated;
+}
+
 void urlEncodeQuery_String(iString *d) {
     iUrl url;
     init_Url(&url, d);
@@ -372,7 +418,7 @@ void urlEncodeQuery_String(iString *d) {
     initRange_String(&query, url.query);
     iString *encQuery = urlEncode_String(&query); /* fully encoded */
     appendCStr_String(&encoded, "?");
-    append_String(&encoded, encQuery);    
+    append_String(&encoded, encQuery);
     delete_String(encQuery);
     deinit_String(&query);
     appendRange_String(&encoded, (iRangecc){ url.query.end, constEnd_String(d) });
@@ -389,7 +435,7 @@ iBool isKnownScheme_Rangecc(iRangecc scheme) {
         if (equalCase_Rangecc(scheme, uriSchemes[i])) {
             return iTrue;
         }
-    }    
+    }
     return iFalse;
 }
 
@@ -414,8 +460,12 @@ const iString *absoluteUrl_String(const iString *d, const iString *urlMaybeRelat
         /* Probably not an URL, so we can't make this absolute. */
         return urlMaybeRelative;
     }
+    if (equalCase_Rangecc(rel.scheme, "misfin")) {
+        /* Use as-is. */
+        return urlMaybeRelative;
+    }
     const iBool isRelative = !isDef_(rel.host);
-    iRangecc scheme = range_CStr("gemini");
+    iRangecc scheme = isDef_(orig.scheme) ? orig.scheme : range_CStr("gemini");
     if (isDef_(rel.scheme)) {
         scheme = rel.scheme;
     }
@@ -483,6 +533,9 @@ const iString *absoluteUrl_String(const iString *d, const iString *urlMaybeRelat
 }
 
 iBool isLikelyUrl_String(const iString *d) {
+    if (endsWith_String(d, ":")) {
+        return iFalse;
+    }
     /* Guess whether a human intends the string to be an URL. This is supposed to be fuzzy;
        not completely per-spec: a) begins with a scheme; b) has something that looks like a
        hostname */
@@ -497,6 +550,10 @@ iBool isLikelyUrl_String(const iString *d) {
     return likelyUrl;
 }
 
+iBool isTitanUrl_String(const iString *d) {
+    return equalCase_Rangecc(urlScheme_String(d), "titan");
+}
+
 static iBool equalPuny_(const iString *d, iRangecc orig) {
     if (!endsWith_String(d, "-")) {
         return iFalse; /* This is a sufficient condition? */
@@ -593,68 +650,27 @@ const iString *findContainerArchive_Path(const iString *path) {
     return NULL;
 }
 
+const char *mimetypes[] = {
+    ".gmi", "text/gemini",
+    ".gemini", "text/gemini",
+    ".pem", "application/x-pem-file",
+    ".gpub", "application/gpub+zip",
+    ".wav", "audio/wave", /* overrides mimetypes.i entry */
+    ".fontpack", mimeType_FontPack,
+    ".md", "text/markdown",
+    ".mdown", "text/markdown",
+    ".markdn", "text/markdown",
+    ".markdown", "text/markdown",
+#include "mimetypes.i"
+#include "plaintext.i"
+    NULL, NULL
+};
+
 const char *mediaTypeFromFileExtension_String(const iString *d) {
-    if (endsWithCase_String(d, ".gmi") || endsWithCase_String(d, ".gemini")) {
-        return "text/gemini; charset=utf-8";
-    }
-    else if (endsWithCase_String(d, ".pem")) {
-        return "application/x-pem-file";
-    }
-    else if (endsWithCase_String(d, ".zip")) {
-        return "application/zip";
-    }
-    else if (endsWithCase_String(d, ".gpub")) {
-        return "application/gpub+zip";
-    }
-    else if (endsWithCase_String(d, ".fontpack")) {
-        return mimeType_FontPack;
-    }
-    else if (endsWithCase_String(d, ".ttf")) {
-        return "font/ttf";
-    }
-    else if (endsWithCase_String(d, ".xml")) {
-        return "text/xml";
-    }
-    else if (endsWithCase_String(d, ".png")) {
-        return "image/png";
-    }
-    else if (endsWithCase_String(d, ".webp")) {
-        return "image/webp";
-    }
-    else if (endsWithCase_String(d, ".jpg") || endsWithCase_String(d, ".jpeg")) {
-        return "image/jpeg";
-    }
-    else if (endsWithCase_String(d, ".gif")) {
-        return "image/gif";
-    }
-    else if (endsWithCase_String(d, ".wav")) {
-        return "audio/wave";
-    }
-    else if (endsWithCase_String(d, ".ogg")) {
-        return "audio/ogg";
-    }
-    else if (endsWithCase_String(d, ".mp3")) {
-        return "audio/mpeg";
-    }
-    else if (endsWithCase_String(d, ".mid")) {
-        return "audio/midi";
-    }
-    else if (endsWithCase_String(d, ".md") ||
-             endsWithCase_String(d, ".markdown") ||
-             endsWithCase_String(d, ".mdown") ||
-             endsWithCase_String(d, ".markdn")) {
-        return "text/markdown";
-    }
-    else if (endsWithCase_String(d, ".txt") ||
-             endsWithCase_String(d, ".ini") ||
-             endsWithCase_String(d, ".md") ||
-             endsWithCase_String(d, ".c") ||
-             endsWithCase_String(d, ".h") ||
-             endsWithCase_String(d, ".cc") ||
-             endsWithCase_String(d, ".hh") ||
-             endsWithCase_String(d, ".cpp") ||
-             endsWithCase_String(d, ".hpp")) {
-        return "text/plain";
+    for (int i = 0; mimetypes[i]; i += 2) {
+        if (endsWithCase_String(d, mimetypes[i])) {
+            return mimetypes[i + 1];
+        }
     }
     return "application/octet-stream";
 }
@@ -689,7 +705,7 @@ static void replaceAllChars_String_(iString *d, char c, const char *replacement)
         remove_Block(&d->chars, pos, 1);
         insertData_Block(&d->chars, pos, replacement, repLen);
         pos += repLen;
-    }    
+    }
 }
 
 void urlEncodeSpaces_String(iString *d) {
@@ -751,7 +767,7 @@ const iString *canonicalUrl_String(const iString *d) {
             canon = copy_String(d);
         }
         urlEncodeSpaces_String(canon);
-    }    
+    }
     return canon ? collect_String(canon) : d;
 }
 
@@ -868,6 +884,14 @@ static const struct {
       { 0x1f645, /* no good */
         "${error.certverify}",
         "${error.certverify.msg}" } },
+    { proxyCertificateExpired_GmStatusCode,
+      { 0x1f4C6, /* calendar */
+        "${error.proxyexpired}",
+        "${error.proxyexpired.msg}" } },
+    { proxyCertificateNotVerified_GmStatusCode,
+      { 0x1f645, /* no good */
+        "${error.proxyverify}",
+        "${error.proxyverify.msg}" } },
     { ansiEscapes_GmStatusCode,
       { 0x1f5b3, /* old computer */
         "${error.ansi}",
@@ -876,6 +900,10 @@ static const struct {
       { 0x1f520, /* ABCD */
         "${error.glyphs}",
         "${error.glyphs.msg}" } },
+    { unsupportedMimeTypeShownAsUtf8_GmStatusCode,
+      { 0x1f524, /* abc */
+        "${error.showutf8}",
+        "${error.showutf8.msg}" } },
     { temporaryFailure_GmStatusCode,
       { 0x1f50c, /* electric plug */
         "${error.temporary}",
